<?php

/*
 * @package RS-MINI
 * @copyright (c) 2015 Alexey Glumov aka Rio-Shaman (support@rio-shaman.ru)
 * @license GNU General Public License version 2; see LICENSE.txt
 *
 */

namespace core;

if(!defined('RS-MINI')) die();

/*
 * application v 1.0
 * 
 * Класс стартует систему
 *
 */

class application
{
    /*
     * объект с картой сайта
     *
     * @var         - object
     * @access      - private
     */
    
    private static $map;

    /*
     * объект дома
     *
     * @var         - object
     * @access      - private
     */
    
    private static $dom;

    /*
     * главный метод. запуск системы
     * 
     * @access  - public
     * 
     */
     
    public static function run()
    {
        // поднимаем некоторые классы ядра
        // объект по работе с структурой сайта
        self::$map = new map();
        // объект прослойка между контроллерами и вьюшкой
        self::$dom = new dom();
        
        // определяем кабинет
        if (!room::setRoom())
            self::notFound();
            
        // смотрим какой определен кабинет
        ?><xmp><?
        var_dump(room::getName());
        var_dump(CSS);
        var_dump(IMG);
        var_dump(JS);
        
        var_dump(CONTENT);
        var_dump(SHORT_CONTENT);
        ?></xmp><?
        
        die();
        
        // получаем открываемую страницу
        if (!self::$map->getPage())
            self::notFound();
            
        // мероприятия по объеденению унаследованных контроллеров с конечной страницей
        self::unionRun();
        
        // запускаем контроллеры
        self::controllerRun();
    }
    
    /*
     * метод стартует контроллеры
     * 
     * @access - private
     * 
     */

    private static function controllerRun()
    {
        // создаем узел хлебных крошек
        $breadCrumbs = self::$dom->addNode('breadcrumbs');
        // массив с псевдонимами страниц для формирования пути
        $path        = array();
        
        // массив с контроллерами которые необходимо будет запустить
        $runList     = array();

        // листаем страницы
        foreach (self::$map->pageList as $key => $page) {

            // список блоков контроллеров
            $blockList = array(
                'top',
                'center',
                'bottom'
            );
            
            // создаем узел в хлебных крошках
            $pageNode = $breadCrumbs->addNode('page');
            // созраняем название страницы
            $caption  = $page->pageName;
            // сохраняем псевдоним в массиве пути
            $path[]   = $page->pageAlias;

            // если на странице контроллеры
            if ($page->controllerList) {
                // листаем все возможные блоки
                foreach ($blockList as $blockAlias) {
                    // листаем контроллеры страницы
                    foreach ($page->controllerList as $block => $controllerList) {
                        // если листаемый блок совпадает с блоком контроллеров страницы - то разбираем контроллеры данного блока
                        if ($blockAlias == $block) {
                            // сначало проглядываем наследуемые контроллеры (потом подключенные на прямую)
                            for ($inh = 0; $inh <= 1; $inh++) {
                                // листаем контроллеры
                                foreach ($controllerList as $controller) {

                                    // проверка наследия (первая итерация ищет только наследованные контроллры)
                                    // при $inh == 0 запускаются только унаследованные контроллеры
                                    // при $inh == 1 запускаются подключенные на прямую
                                    if ($controller->inheritFlag != ( ($inh == 0) ? TRUE : FALSE ))
                                        continue;

                                    // пропускаем отключенные контроллеры
                                    if ($controller->getParam('active') == 'disable')
                                        continue;

                                    // подставляем в путь допол. папку
                                    // бьем имя контроллера на массив
                                    $controllerClass                                  = explode('.', $controller->controllerName);
                                    // отрезаем два первых элемента массива
                                    $controllerSlice                                  = array_slice($controllerClass, 2);
                                    // во второй элемент массива вставляем слово controller
                                    $controllerClass[2]                               = 'controller';
                                    // вертаем в массив оставшие элементы после отработки функции array_slice
                                    for ($i = 0; isset($controllerSlice[$i]); $i++)
                                        $controllerClass[($i + 3)]                    = $controllerSlice[$i];

                                    // тем самым получаем полный путь к классу
                                    // пример: было pub.test.intro а стало pub\test\controller\intro
                                    $controllerClass                                  = implode('\\', $controllerClass);

                                    // записываем в параметры инфу о странице
                                    $controller
                                        ->setParam('page_id',     $page->pageID)
                                        ->setParam('page_alias',  $page->pageAlias)
                                        ->setParam('page_parent', $page->pageParent)
                                    ;

                                    // создаем объект контроллера
                                    $obj = new $controllerClass(self::$dom, $controller->getParams());
                                    
                                    // регистрируем динамичный алиас и его значение в
                                    // статичном свойстве абстрактного класса контроллера
                                    // если реальный алиас страницы row_id а в адресную строку было вбито
                                    // /123/ то в свойстве dynamicValue абстракного класса abstractcontroller
                                    // будет зарегистрирован ключ row_id с значением 123
                                    // dynamicValue['row_id'] = '123'
                                    if ($page->pageType == 'dynamic')
                                        $obj->setDynamicValue($page->pageDynamicAlias, $page->pageAlias);

                                    // проверяем на валидацию
                                    if (!$obj->validate())
                                        self::notFound();

                                    // получаем заголовок
                                    $caption = $obj->getCaption($caption);

                                    // если листается последняя страница
                                    if ($key == (count(self::$map->pageList) - 1))
                                        // сохраняем данные для запуска (запуск ниже)
                                        $runList[] = array(
                                            'obj'           => $obj,
                                            'controller'    => $controller
                                        );

                                }
                            }
                        }
                    }
                }
            }
            
            // атрибуты хлебных крошек
            $pageNode->addAttr('caption',  $caption);
            $pageNode->addAttr('path',     (count($path) > 1) ? implode('/', $path) . '/' : '/');

            // если листается последняя страница, запускаем контроллеры страницы и шаблонизатор
            if ($key == (count(self::$map->pageList) - 1)) {
                
                // объявляем констранту с заголовком открытой страницы
                define('PAGECAPTION', $caption);
                
                // массив с шаблонами контроллеров
                $tplList = array();
            
                // если в массиве runList есть контроллеры для запуска
                if (!empty($runList)) {
                    // листаем контроллеры
                    foreach ($runList as $data) {

                        // стартуем контроллер
                        if (!$data['obj']->run())
                            self::notFound();
                            
                        // если среди парамеров есть параметр шаблона
                        if(!is_null($data['controller']->getParam('tpl'))) {
                        
                            // пропускаем отключенные контроллеры
                            if ($data['controller']->getParam('tpl') == 'disable')
                                continue;
                        
                            // подставляем в путь доп папку
                            // бьем имя шаблона на массив
                            $tplClass                                      = explode('.', $data['controller']->getParam('tpl'));
                            // отрезаем первые два элемента
                            $tplSlice                                      = array_slice($tplClass, 2);
                            // в третий элемент массива вставляем слово view
                            $tplClass[2]                                   = 'view';
                            // вертаем в массив оставшие элементы после отработки функции array_slice
                            for ($i = 0; isset($tplSlice[$i]); $i++)
                                $tplClass[($i + 3)]                        = $tplSlice[$i];
                                
                            // тем самым получаем полный путь к классу
                            // пример: было pub.test.intro а стало pub\test\view\intro
                            $tplClass                                      = implode('\\', $tplClass);
                            // создаем объект в массиве ключем которого является
                            // имя шаблона
                            $tplList[$data['controller']->getParam('tpl')] = new $tplClass(self::$dom);
                        }
                    }
                }
                
                // запускаем шаблонизатор
                self::viewRun($tplList, $page);
            }
        }
    }
    
    /*
     * метод стартует шаблонизатор
     * 
     * @access  - private
     * 
     * @param array  tplList  - массив с шаблонами
     * @param object page     - объект последний (открываемой) страницы
     */
     
    private static function viewRun($tplList, $page)
    {
        // запускаем представление
        view::create($tplList, $page->pageView, self::$dom)->run();
    }
    
    /*
     * метод добавляет унаследованные контроллеры в вложенные страницы
     * 
     * @access - private
     * 
     */

    private static function unionRun()
    {
        // массив с контроллерами которые отмечены как наследуемые.
        // при первой итерации массив пуст
        $inherit = array();

        // листаем страницы
        for ($key = 0; isset(self::$map->pageList[$key]); $key++) {
            // если массив хранящий контроллеры которые передаются по наследству не пуст
            // то запускаем спец метод который запишет унаследованные контроллеры
            // в обрабатываемую страницу (при первой итерации этот массив пуст)
            if (!empty($inherit)) {
                // сливаем контроллеры родительской страницы и текцщей
                self::$map->pageList[$key]->unionController($inherit);
                // сбрасываем массив
                $inherit = array();
            }
            
            // если на странице есть контроллеры
            if (self::$map->pageList[$key]->controllerList)
                // листаем блоки контроллеров
                foreach (self::$map->pageList[$key]->controllerList as $block => $controllerList)
                    // листаем контроллеры блока
                    foreach ($controllerList as $controller) {
                        // если итерация страниц не последняя (т.е. если страница не последняя)
                        if ($key != (count(self::$map->pageList) - 1))
                            // если контроллер отмечен как наследуемый
                            if ($controller->controllerInherit == 'true')
                                // заносим объект контроллера в массив inherit
                                // для передачи этого объекта по наследству в
                                // дочернию страницу
                                $inherit[$block][] = $controller;
                    }
        }
    }
    
    /*
     * метод выводит 404 ошибку
     * 
     * @access - private
     */

    private static function notFound()
    {
        header('HTTP/1.0 404 Not Fount');
        die('404 error');
    }
}
